@@ -1,6 +1,6 @@
 ;;;; scrutinizer.scm - The CHICKEN Scheme compiler (local flow analysis)
 ;
-; Copyright (c) 2009-2016, The CHICKEN Team
+; Copyright (c) 2009-2017, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -26,9 +26,10 @@
 
 (declare
   (unit scrutinizer)
-  (hide specialize-node! specialization-statistics
+  (hide specialize-node! specialization-statistics mutate-node! node-mutations
 	procedure-type? named? procedure-result-types procedure-argument-types
 	noreturn-type? rest-type procedure-name d-depth
+	source-node source-node-tree node-line-number node-debug-info
 	noreturn-procedure-type? trail trail-restore walked-result 
 	multiples procedure-arguments procedure-results typeset-min
 	smash-component-types! generate-type-checks! over-all-instantiations
@@ -123,22 +124,39 @@
 (define (walked-result n)
   (first (node-parameters n)))		; assumes ##core#the/result node
 
-(define (node-line-number n)
-  (case (node-class n)
-    ((##core#call)
-     (let ((params (node-parameters n)))
-       (and (pair? (cdr params))
-	    (pair? (cadr params)) ; debug-info has line-number information?
-	    (source-info->line (cadr params)))))
-    ((##core#typecase)
-     (car (node-parameters n)))
-    (else #f)))
+(define (type-always-immediate? t)
+  (cond ((pair? t)
+	 (case (car t)
+	   ((or) (every type-always-immediate? (cdr t)))
+	   ((forall) (type-always-immediate? (third t)))
+	   (else #f)))
+	((memq t '(eof null fixnum char boolean undefined)) #t)
+	(else #f)))
+
+(define (node-source-prefix n)
+  (let ((line (node-line-number n)))
+    (if (not line) "" (sprintf "(~a) " line))))
+
+(define (location-name loc)
+  (define (lname loc1)
+    (if loc1
+	(sprintf "procedure `~a'" (real-name loc1))
+	"unknown procedure"))
+  (cond ((null? loc) "at toplevel:\n  ")
+	((null? (cdr loc))
+	 (sprintf "in toplevel ~a:\n  " (lname (car loc))))
+	(else
+	 (let rec ((loc loc))
+	   (if (null? (cdr loc))
+	       (location-name loc)
+	       (sprintf "in local ~a,\n  ~a" (lname (car loc)) (rec (cdr loc))))))))
 
 (define (scrutinize node db complain specialize)
   (let ((blist '())			; (((VAR . FLOW) TYPE) ...)
 	(aliased '())
 	(noreturn #f)
 	(dropped-branches 0)
+	(assigned-immediates 0)
 	(errors #f)
 	(safe-calls 0))
 
@@ -219,15 +237,25 @@
 	    ((memq t '(* boolean false undefined noreturn)) #f)
 	    (else #t)))
 
-    (define (always-true t loc x)
-      (let ((f (always-true1 t)))
-	(when f
-	  (report-notice
-	   loc
-	   "expected a value of type boolean in conditional, but \
-	    was given a value of type `~a' which is always true:~%~%~a"
-	   t (pp-fragment x)))
-	f))
+    (define (always-true if-node test-node t loc)
+      (and-let* ((_ (always-true1 t)))
+	(report-notice
+	 loc "~aexpected a value of type boolean in conditional, but \
+	 was given a value of type `~a' which is always true:~%~%~a"
+	 (node-source-prefix test-node) t (pp-fragment if-node))
+	#t))
+
+    (define (always-false if-node test-node t loc)
+      (and-let* ((_ (eq? t 'false)))
+	(report-notice
+	 loc "~ain conditional, test expression will always return false:~%~%~a"
+	 (node-source-prefix test-node) (pp-fragment if-node))
+	#t))
+
+    (define (always-immediate var t loc)
+      (and-let* ((_ (type-always-immediate? t)))
+	(d "assignment to var ~a in ~a is always immediate" var loc)
+	#t))
 
     (define (single node what tv loc)
       (if (eq? '* tv)
@@ -263,28 +291,10 @@
       (set! errors #t)
       (apply report loc msg args))
 
-    (define (node-source-prefix n)
-      (let ((line (node-line-number n)))
-       (if (not line) "" (sprintf "(~a) " line))))
-
-    (define (location-name loc)
-      (define (lname loc1)
-	(if loc1
-	    (sprintf "procedure `~a'" (real-name loc1))
-	    "unknown procedure"))
-      (cond ((null? loc) "at toplevel:\n  ")
-	    ((null? (cdr loc))
-	     (sprintf "in toplevel ~a:\n  " (lname (car loc))))
-	    (else
-	     (let rec ((loc loc))
-	       (if (null? (cdr loc))
-		   (location-name loc)
-		   (sprintf "in local ~a,\n  ~a" (lname (car loc)) (rec (cdr loc))))))))
-
     (define add-loc cons)
 
     (define (fragment x)
-      (let ((x (build-expression-tree x)))
+      (let ((x (build-expression-tree (source-node-tree x))))
 	(let walk ((x x) (d 0))
 	  (cond ((atom? x) (##sys#strip-syntax x))
 		((>= d +fragment-max-depth+) '...)
@@ -373,6 +383,7 @@
 					      (specialize-node!
 					       node (cdr args)
 					       `(let ((#(tmp) #(1))) '#t))
+					      (set! r '(true))
 					      (set! op (list pn pt))))
 					   ((begin
 					      (trail-restore trail0 typeenv)
@@ -386,6 +397,7 @@
 					      (specialize-node!
 					       node (cdr args)
 					       `(let ((#(tmp) #(1))) '#f))
+					      (set! r '(false))
 					      (set! op (list pt `(not ,pt)))))
 					   (else (trail-restore trail0 typeenv)))))
 			     ((and specialize (get-specializations pn)) =>
@@ -492,10 +504,13 @@
 			   (a (third subs))
 			   (nor0 noreturn))
 		      (cond
-			((and (always-true rt loc n) specialize)
-			 ;; drop branch and re-walk updated node
+			((and (always-true n tst rt loc) specialize)
 			 (set! dropped-branches (add1 dropped-branches))
-			 (copy-node! (build-node-graph `(let ((,(gensym) ,tst)) ,c)) n)
+			 (mutate-node! n `(let ((,(gensym) ,tst)) ,c))
+			 (walk n e loc dest tail flow ctags))
+			((and (always-false n tst rt loc) specialize)
+			 (set! dropped-branches (add1 dropped-branches))
+			 (mutate-node! n `(let ((,(gensym) ,tst)) ,a))
 			 (walk n e loc dest tail flow ctags))
 			(else
 			 (let* ((r1 (walk c e loc dest tail (cons (car tags) flow) #f))
@@ -675,6 +690,11 @@
 				   (set-cdr! (car bl) t)
 				   (loop (cdr bl) (eq? fl (cdaar bl)))))
 				(else (loop (cdr bl) f))))))
+
+		    (when (always-immediate var rt loc)
+		      (set! assigned-immediates (add1 assigned-immediates))
+		      (set-cdr! params '(#t)))
+
 		    '(undefined)))
 		 ((##core#primitive ##core#inline_ref) '*)
 		 ((##core#call)
@@ -784,7 +804,7 @@
 						 '##compiler#special-result-type))
 					   => (lambda (srt)
 						(dd "  hardcoded special result-type: ~a" var)
-						(set! r (srt n args r))))))))
+						(set! r (srt n args loc r))))))))
 			      subs
 			      (cons 
 			       fn
@@ -836,7 +856,7 @@
 					  (append (type-typeenv (car types)) typeenv)
 					  #t)
 			     ;; drops exp
-			     (copy-node! (car subs) n)
+			     (mutate-node! n (car subs))
 			     (walk n e loc dest tail flow ctags))
 			    (else
 			     (trail-restore trail0 typeenv)
@@ -864,6 +884,8 @@
 	(debugging '(o e) "safe calls" safe-calls))
       (when (positive? dropped-branches)
 	(debugging '(o e) "dropped branches" dropped-branches))
+      (when (positive? assigned-immediates)
+	(debugging '(o e) "assignments to immediate values" assigned-immediates))
       (when errors
 	(quit "some variable types do not satisfy strictness"))
       rn)))
@@ -885,11 +907,16 @@
 	   (dd "  smashing `~s' in ~a" (caar lst) where)
 	   (change! 'vector)
 	   (car t))
+	  ((vector)
+	   (dd "  smashing `~s' in ~a" (caar lst) where)
+	   ;; (vector x y z) => (vector * * *)
+	   (change! (cons 'vector (map (constantly '*) (cdr t))))
+	   (car t))
 	  ((list-of list)
 	   (dd "  smashing `~s' in ~a" (caar lst) where)
 	   (change! '(or pair null))
 	   (car t))
-	  ((pair vector)
+	  ((pair)
 	   (dd "  smashing `~s' in ~a" (caar lst) where)
 	   (change! (car t))
 	   (car t))
@@ -1821,6 +1848,47 @@
        db)
       (print "; END OF FILE"))))
 
+;;
+;; Source node tracking
+;;
+;; Nodes are mutated in place during specialization, which may lose line
+;; number information if, for example, a node is changed from a
+;; ##core#call to a class without debug info. To preserve line numbers
+;; and allow us to print fragments of the original source, we maintain a
+;; side table of mappings from mutated nodes to copies of the originals.
+;;
+
+(define node-mutations '())
+
+(define (mutate-node! node expr)
+  (set! node-mutations (alist-update! node (copy-node node) node-mutations))
+  (copy-node! (build-node-graph expr) node))
+
+(define (source-node n #!optional (k values))
+  (let ((orig (alist-ref n node-mutations eq?)))
+    (if (not orig) (k n) (source-node orig k))))
+
+(define (source-node-tree n)
+  (source-node
+   n
+   (lambda (n*)
+     (make-node (node-class n*)
+		(node-parameters n*)
+		(map source-node (node-subexpressions n*))))))
+
+(define (node-line-number n)
+  (node-debug-info (source-node n)))
+
+(define (node-debug-info n)
+  (case (node-class n)
+    ((##core#call)
+     (let ((params (node-parameters n)))
+       (and (pair? (cdr params))
+	    (pair? (cadr params)) ; debug-info has line-number information?
+	    (source-info->line (cadr params)))))
+    ((##core#typecase)
+     (car (node-parameters n)))
+    (else #f)))
 
 ;; Mutate node for specialization
 
@@ -1848,8 +1916,7 @@
 	    ((not (pair? x)) x)
 	    ((eq? 'quote (car x)) x)	; to handle numeric constants
 	    (else (cons (subst (car x)) (subst (cdr x))))))
-    (let ((spec (subst template)))
-      (copy-node! (build-node-graph spec) node))))
+    (mutate-node! node (subst template))))
 
 
 ;;; Type-validation and -normalization
@@ -2118,7 +2185,7 @@
      (##sys#put! 'name '##compiler#special-result-type handler))))
 
 (define-special-case ##sys#make-structure
-  (lambda (node args rtypes)
+  (lambda (node args loc rtypes)
     (or (and-let* ((subs (node-subexpressions node))
                    ((>= (length subs) 2))
                    (arg1 (second subs))
@@ -2133,22 +2200,66 @@
 	rtypes)))
 
 (let ()
-  (define (vector-ref-result-type node args rtypes)
-    (or (and-let* ((subs (node-subexpressions node))
-                   ((= (length subs) 3))
-                   (arg1 (walked-result (second args)))
-                   ((pair? arg1))
-                   ((eq? 'vector (car arg1)))
-                   (index (third subs))
-                   ((eq? 'quote (node-class index)))
-                   (val (first (node-parameters index)))
-                   ((fixnum? val))
-                   ((>= val 0))
-                   ((< val (length (cdr arg1))))) ;XXX could warn on failure (but needs location)
-          (list (list-ref (cdr arg1) val)))
+  ;; TODO: Complain argument not available here, so we can't use the
+  ;; standard "report" defined above.  However, ##sys#enable-warnings
+  ;; and "complain" (do-scrutinize) are always true together, except
+  ;; that "complain" will be false while ##sys#enable-warnings is true
+  ;; on "no-usual-integrations", so perhaps get rid of "complain"?
+  (define (report loc msg . args)
+    (warning
+     (conc (location-name loc)
+	   (sprintf "~?" msg (map unrename-type args)))))
+
+  (define (known-length-vector-index node args loc expected-argcount)
+    (and-let* ((subs (node-subexpressions node))
+	       ((= (length subs) (add1 expected-argcount)))
+	       (arg1 (walked-result (second args)))
+	       ((pair? arg1))
+	       ((eq? 'vector (car arg1)))
+	       (index (third subs))
+	       ((eq? 'quote (node-class index)))
+	       (val (first (node-parameters index)))
+	       ((fixnum? val)) ; Standard type warning otherwise
+	       (vector-length (length (cdr arg1))))
+      (if (and (>= val 0) (< val vector-length))
+	  val
+	  (begin
+	    (report
+	     loc "~ain procedure call to `~s', index ~a out of range \
+                   for vector of length ~a"
+	     (node-source-prefix node)
+	     ;; TODO: It might make more sense to use "pname" here
+	     (first (node-parameters (first subs))) val vector-length)
+	    #f))))
+
+  ;; These are a bit hacky, since they mutate the node.  These special
+  ;; cases are really only intended for determining result types...
+  (define (vector-ref-result-type node args loc rtypes)
+    (or (and-let* ((index (known-length-vector-index node args loc 2))
+		   (arg1 (walked-result (second args)))
+		   (vector (second (node-subexpressions node))))
+	  (mutate-node! node `(##sys#slot ,vector ',index))
+	  (list (list-ref (cdr arg1) index)))
 	rtypes))
+
   (define-special-case vector-ref vector-ref-result-type)
-  (define-special-case ##sys#vector-ref vector-ref-result-type))
+  (define-special-case ##sys#vector-ref vector-ref-result-type)
+
+  (define-special-case vector-set!
+    (lambda (node args loc rtypes)
+      (or (and-let* ((index (known-length-vector-index node args loc 3))
+		     (subs (node-subexpressions node))
+		     (vector (second subs))
+		     (new-value (fourth subs))
+		     (new-value-type (walked-result (fourth args)))
+		     (setter (if (type-always-immediate? new-value-type)
+				 '##sys#setislot
+				 '##sys#setslot)))
+	    (mutate-node! node `(,setter ,vector ',index ,new-value))
+	    '(undefined))
+	  rtypes))))
+
+;; TODO: Also special-case vector-length?  Makes little sense though.
 
 
 ;;; List-related special cases
@@ -2158,12 +2269,18 @@
 ;   list-ref, list-tail, drop, take
 
 (let ()
+  ;; See comment in vector (let) just above this
+  (define (report loc msg . args)
+    (warning
+     (conc (location-name loc)
+	   (sprintf "~?" msg (map unrename-type args)))))
 
   (define (list-or-null a)
     (if (null? a) 'null `(list ,@a)))
 
   ;; Split a list or pair type form at index i, calling k with the two
   ;; sections of the type or returning #f if it doesn't match that far.
+  ;; Note that "list-of" is handled by "forall" entries in types.db
   (define (split-list-type l i k)
     (cond ((not (pair? l))
 	   (and (fx= i 0) (eq? l 'null) (k l l)))
@@ -2184,17 +2301,43 @@
 		   (else #f))))
 	  (else #f)))
 
+  ;; canonicalize-list-type will have taken care of converting (pair
+  ;; (pair ...)) to (list ...) or (list-of ...) for proper lists.
+  (define (proper-list-type-length t)
+    (cond ((eq? t 'null) 0)
+	  ((and (pair? t) (eq? (car t) 'list)) (length (cdr t)))
+	  (else #f)))
+
   (define (list+index-call-result-type-special-case k)
-    (lambda (node args rtypes)
+    (lambda (node args loc rtypes)
       (or (and-let* ((subs (node-subexpressions node))
 		     ((= (length subs) 3))
 		     (arg1 (walked-result (second args)))
 		     (index (third subs))
 		     ((eq? 'quote (node-class index)))
 		     (val (first (node-parameters index)))
-		     ((fixnum? val))
-		     ((>= val 0)))
-	    (split-list-type arg1 val k))
+		     ((fixnum? val))) ; Standard type warning otherwise
+	    ;; TODO: It might make sense to use "pname" when reporting
+	    (cond ((negative? val)
+		   ;; Negative indices should always generate a warning
+		   (report
+		    loc "~ain procedure call to `~s', index ~a is \
+                        negative, which is never valid"
+		    (node-source-prefix node)
+		    (first (node-parameters (first subs))) val)
+		   #f)
+		  ((split-list-type arg1 val k))
+		  ;; Warn only if it's a known proper list.  This avoids
+		  ;; false warnings due to component smashing.
+		  ((proper-list-type-length arg1) =>
+		   (lambda (length)
+		     (report
+		      loc "~ain procedure call to `~s', index ~a out of \
+                        range for proper list of length ~a"
+		      (node-source-prefix node)
+		      (first (node-parameters (first subs))) val length)
+		     #f))
+		  (else #f)))
 	  rtypes)))
 
   (define-special-case list-ref
@@ -2216,27 +2359,27 @@
      (lambda (_ result-type) (list result-type)))))
 
 (define-special-case list
-  (lambda (node args rtypes)
+  (lambda (node args loc rtypes)
     (if (null? (cdr args))
 	'(null)
 	`((list ,@(map walked-result (cdr args)))))))
 
 (define-special-case ##sys#list
-  (lambda (node args rtypes)
+  (lambda (node args loc rtypes)
     (if (null? (cdr args))
 	'(null)
 	`((list ,@(map walked-result (cdr args)))))))
 
 (define-special-case vector
-  (lambda (node args rtypes)
+  (lambda (node args loc rtypes)
     `((vector ,@(map walked-result (cdr args))))))
 
 (define-special-case ##sys#vector
-  (lambda (node args rtypes)
+  (lambda (node args loc rtypes)
     `((vector ,@(map walked-result (cdr args))))))
 
 (define-special-case reverse
-  (lambda (node args rtypes)
+  (lambda (node args loc rtypes)
     (or (and-let* ((subs (node-subexpressions node))
 		   ((= (length subs) 2))
 		   (arg1 (walked-result (second args)))
@@ -2245,6 +2388,58 @@
 	  `((list ,@(reverse (cdr arg1)))))
 	rtypes)))
 
+(let ()
+  ;; See comment in vector (let)
+  (define (report loc msg . args)
+    (warning
+     (conc (location-name loc)
+	   (sprintf "~?" msg (map unrename-type args)))))
+
+  (define (append-special-case node args loc rtypes)
+    (define (potentially-proper-list? l) (match-types l 'list '()))
+
+    (define (derive-result-type)
+      (let lp ((arg-types (cdr args))
+	       (index 1))
+	(if (null? arg-types)
+	    'null
+	    (let ((arg1 (walked-result (car arg-types))))
+	      (cond
+	       ((and (pair? arg1) (eq? (car arg1) 'list))
+		(and-let* ((rest-t (lp (cdr arg-types) (add1 index))))
+		  ;; decanonicalize, then recanonicalize to make it
+		  ;; easy to append a variety of types.
+		  (canonicalize-list-type
+		   (foldl (lambda (rest t) `(pair ,t ,rest))
+			  rest-t (reverse (cdr arg1))))))
+
+	       ((and (pair? arg1) (eq? (car arg1) 'list-of))
+		(and-let* ((rest-t (lp (cdr arg-types) (add1 index))))
+		  ;; list-of's length unsurety is "contagious"
+		  (simplify-type `(or ,arg1 ,rest-t))))
+
+	       ;; TODO: (append (pair x (pair y z)) lst) =>
+	       ;; (pair x (pair y (or z lst)))
+	       ;; This is trickier than it sounds!
+
+	       (else
+		;; The final argument may be an atom or improper list
+		(unless (or (null? (cdr arg-types))
+			    (potentially-proper-list? arg1))
+		  (report
+		   loc "~ain procedure call to `~s', argument #~a is \
+			of type ~a but expected a proper list"
+		   (node-source-prefix node)
+		   (first (node-parameters
+			   (first (node-subexpressions node))))
+		   index arg1))
+		#f))))))
+    (cond ((derive-result-type) => list)
+	  (else rtypes)))
+
+  (define-special-case append append-special-case)
+  (define-special-case ##sys#append append-special-case))
+
 ;;; Special cases for make-list/make-vector with a known size
 ;
 ; e.g. (make-list 3 #\a) => (list char char char)
@@ -2252,7 +2447,7 @@
 (let ()
 
   (define (complex-object-constructor-result-type-special-case type)
-    (lambda (node args rtypes)
+    (lambda (node args loc rtypes)
       (or (and-let* ((subs (node-subexpressions node))
 		     (fill (case (length subs)
 			     ((2) '*)
