@@ -7,7 +7,7 @@ int main()
 {
   static char buffer[ 4096 ];
 
-  void C_toplevel(C_word x, C_word y, C_word z);
+  void C_toplevel(C_word c, C_word *av);
 
   CHICKEN_run((void*)C_toplevel);
   assert(CHICKEN_eval_string_to_string("(oink (make-vector 10 'ok))",
