@@ -322,11 +322,25 @@
 
 (parameterize ((keyword-style #:suffix))
   (assert (string=? "abc:" (symbol->string (with-input-from-string "|abc:|" read))))
-  (assert (string=? "abc" (symbol->string (with-input-from-string "|abc|:" read))))) ; keyword
+  (assert (string=? "abc" (symbol->string (with-input-from-string "|abc|:" read)))) ; keyword
+  (let ((kw (with-input-from-string "|foo bar|:" read)))
+    (assert (eq? kw (with-input-from-string "#:|foo bar|" read)))
+    (assert (string=? "foo bar" (symbol->string kw)))
+    (assert (string=? "foo bar:"
+		      (with-output-to-string (lambda () (display kw)))))
+    (assert (string=? "|foo bar|:"
+		      (with-output-to-string (lambda () (write kw)))))))
 
 (parameterize ((keyword-style #:prefix))
   (assert (string=? "abc" (symbol->string (with-input-from-string ":|abc|" read))))
-  (assert (string=? ":abc" (symbol->string (with-input-from-string "|:abc|" read)))))
+  (assert (string=? ":abc" (symbol->string (with-input-from-string "|:abc|" read))))
+  (let ((kw (with-input-from-string ":|foo bar|" read)))
+    (assert (eq? kw (with-input-from-string "#:|foo bar|" read)))
+    (assert (string=? "foo bar" (symbol->string kw)))
+    (assert (string=? ":foo bar"
+		      (with-output-to-string (lambda () (display kw)))))
+    (assert (string=? ":|foo bar|"
+		      (with-output-to-string (lambda () (write kw)))))))
 
 (assert (eq? '|#:| (string->symbol "#:")))
 (assert-fail (with-input-from-string "#:" read)) ; empty keyword
@@ -353,10 +367,29 @@
   (assert (not (keyword? (with-input-from-string ":abc:" read))))
   (assert (not (keyword? (with-input-from-string "abc:" read)))))
 
-(assert (string=? ":" (symbol->string (with-input-from-string ":" read))))
-(assert (string=? ":" (symbol->string (with-input-from-string ":||" read))))
+(let ((colon-sym (with-input-from-string ":" read)))
+  (assert (symbol? colon-sym))
+  (assert (not (keyword? colon-sym)))
+  (assert (string=? ":" (symbol->string colon-sym))))
+
+;; The next two cases are a bit dubious.  These could also be read as
+;; keywords due to the literal quotation.
+(let ((colon-sym (with-input-from-string ":||" read)))
+  (assert (symbol? colon-sym))
+  (assert (not (keyword? colon-sym)))
+  (assert (string=? ":" (symbol->string colon-sym))))
+
+(let ((colon-sym (with-input-from-string "||:" read)))
+  (assert (symbol? colon-sym))
+  (assert (not (keyword? colon-sym)))
+  (assert (string=? ":" (symbol->string colon-sym))))
+
 (assert-fail (with-input-from-string "#:" read))
 
+(let ((empty-kw (with-input-from-string "#:||" read)))
+  (assert (keyword? empty-kw))
+  (assert (string=? "" (keyword->string empty-kw))))
+
 (assert (keyword? (with-input-from-string "42:" read)))
 (assert (keyword? (with-input-from-string ".:" read)))
 
@@ -554,6 +587,41 @@ A
   (assert (equal? original-error (current-error-port)))
   (assert (equal? original-exception-handler (current-exception-handler))))
 
+;; Re-entering dynamic extent of a parameterize should not reset to
+;; original outer values but remember values when jumping out (another
+;; regression due to #1227, pointed out by Joo ChurlSoo in #1336).
+
+(let ((f (make-parameter 'a))
+      (path '())
+      (g (make-parameter 'g))
+      (c #f))
+  (let ((add (lambda () (set! path (cons (f) path)))))
+    (add)
+    (parameterize ((f 'b)
+		   (g (call-with-current-continuation
+		       (lambda (c0) (set! c c0) 'c))))
+      (add) (f (g)) (add))
+    (f 'd)
+    (add)
+    (if (< (length path) 8)
+	(c 'e)
+	(assert (equal? '(a b c d b e d b e d) (reverse path))))))
+
+(let ((f (make-parameter 'a))
+      (path '())
+      (g (make-parameter 'g))
+      (c #f))
+  (let ((add (lambda () (set! path (cons (f) path)))))
+    (add)
+    (parameterize ((f 'b))
+      (g (call-with-current-continuation (lambda (c0) (set! c c0) 'c)))
+      (add) (f (g)) (add))
+    (f 'd)
+    (add)
+    (if (< (length path) 8)
+	(c 'e)
+	(assert (equal? '(a b c d c e d e e d) (reverse path))))))
+
 ;;; vector and blob limits
 
 (assert-fail (make-blob -1))
