@@ -1,7 +1,7 @@
 ;;;; typematch-tests.scm
 
 
-(use lolevel data-structures)
+(use srfi-1 lolevel data-structures)
 
 
 (define-syntax check
@@ -27,52 +27,59 @@
 
 (define (bar) 42)
 
-(define-syntax m
+(define-syntax type<=
   (er-macro-transformer
    (lambda (x r c)
      (let ((t1 (cadr x))
 	   (t2 (caddr x))
-	   (foo1 (gensym 'foo1))
-	   (foo2 (gensym 'foo2)))
+	   (foo (gensym 'foo)))
        `(begin
 	  (print ',t1 " = " ',t2)
-	  (: ,foo1 (-> ,t1))
-	  (: ,foo2 (-> ,t2))
-	  (define (,foo1) (bar))
-	  (define (,foo2) (bar))
-	  (compiler-typecase (,foo1)
-	    (,t2 'ok))
-	  (print ',t2 " = " ',t1)
-	  (compiler-typecase (,foo2)
-	    (,t1 'ok)))))))
+	  (: ,foo (-> ,t1))
+	  (define (,foo) (bar))
+	  (compiler-typecase (,foo)
+	    (,t2 'ok)))))))
 
-(define-syntax mx
-  (syntax-rules ()
-    ((_ t x) 
-     (compiler-typecase 
-      x
-      (t 'ok)))))
+(define-syntax type>
+  (er-macro-transformer
+   (lambda (x r c)
+     (let ((t1 (cadr x))
+	   (t2 (caddr x))
+	   (foo (gensym 'foo)))
+       `(begin
+	  (print ',t1 " != " ',t2)
+	  (: ,foo (-> ,t1))
+	  (define (,foo) (bar))
+	  (compiler-typecase (,foo)
+	    (,t2 (bomb))
+	    (else 'ok)))))))
+
+(define-syntax m
+  (er-macro-transformer
+   (lambda (x r c)
+     (let ((t1 (cadr x))
+	   (t2 (caddr x)))
+       `(begin
+	  (type<= ,t1 ,t2)
+	  (type<= ,t2 ,t1))))))
 
 (define-syntax mn
   (er-macro-transformer
    (lambda (x r c)
      (let ((t1 (cadr x))
-	   (t2 (caddr x))
-	   (foo1 (gensym 'foo1))
-	   (foo2 (gensym 'foo2)))
+	   (t2 (caddr x)))
        `(begin
-	  (print ',t1 " != " ',t2)
-	  (: ,foo1 (-> ,t1))
-	  (: ,foo2 (-> ,t2))
-	  (define (,foo1) (bar))
-	  (define (,foo2) (bar))
-	  (compiler-typecase (,foo1)
-	    (,t2 (bomb))
-	    (else 'ok))
-	  (print ',t2 " != " ',t1)
-	  (compiler-typecase (,foo2)
-	    (,t1 (bomb))
-	    (else 'ok)))))))
+	  (type> ,t1 ,t2)
+	  (type> ,t2 ,t1))))))
+
+(define-syntax mx
+  (syntax-rules ()
+    ((_ t x)
+     (begin
+       (print 'x " = " 't)
+       (compiler-typecase
+	x
+	(t 'ok))))))
 
 (define-syntax ms
   (er-macro-transformer
@@ -172,7 +179,14 @@
 (checkp pointer-vector? (make-pointer-vector 1) pointer-vector)
 (checkp pointer? (address->pointer 1) pointer)
 
-(mn list null)
+(type<= null list)
+(type<= (list *) list)
+(type<= (vector *) vector)
+
+(type> list null)
+(type> list (list *))
+(type> vector (vector *))
+
 (mn pair null)
 (mn pair list)
 
@@ -206,9 +220,14 @@
 (mx list   (cddr-alike l))
 (mx fixnum (cddr-alike p))
 
+(ms '(1 . 2) '() pair)
 (ms '(1 2) '() pair)
+(ms '(1) '() pair)
+(ms '() '(1) (not pair))
 (ms '() '(1 2) (not pair))
 (ms '() '(1 . 2) (not pair))
+(ms '() '(1 . 2) list)
+(ms '(1 . 2) '() (not list))
 (ms '(1 2) '(1 . 2) (pair * pair))
 (ms '(1 2) '(1 . 2) (pair * list))
 (ms '(1 2) '(1 2 3) (pair * (pair * null)))
@@ -263,6 +282,35 @@
 (mx fixnum (##sys#vector-ref '#(1 2 3.4) 0))
 (mx (vector fixnum float) (vector 1 2.3))
 (mx (list fixnum float) (list 1 2.3))
+(mx (list fixnum float) (list-copy (list 1 2.3)))
+(mx (pair fixnum float) (list-copy (cons 1 2.3)))
+(mx fixnum (list-copy 1))
+(mx fixnum (list-ref (list 1 2.3) 0))
+(mx fixnum (list-ref (cons 1 2.3) 0))
+(mx float (list-ref (list 1 2.3) 1))
+(mx (list fixnum float) (list-tail (list 1 2.3) 0))
+(mx (pair fixnum float) (list-tail (cons 1 2.3) 0))
+(mx (list float) (list-tail (list 1 2.3) 1))
+(mx float (list-tail (cons 1 2.3) 1))
+(mx null  (list-tail (list 1 2.3) 2))
+(mx (list fixnum float) (drop (list 1 2.3) 0))
+(mx (pair fixnum float) (drop (cons 1 2.3) 0))
+(mx (list float) (drop (list 1 2.3) 1))
+(mx float (drop (cons 1 2.3) 1))
+(mx null (drop (list 1 2.3) 2))
+(mx null (take (list 1 2.3) 0))
+(mx null (take (cons 1 2.3) 0))
+(mx (list fixnum) (take (list 1 2.3) 1))
+(mx (list fixnum) (take (cons 1 2.3) 1))
+(mx (list fixnum float) (take (list 1 2.3) 2))
+(mx (list * *) (make-list 2))
+(mx (list string string) (make-list 2 "a"))
+(mx (vector * *) (make-vector 2))
+(mx (vector string string) (make-vector 2 "a"))
+(mx null (reverse '()))
+(mx list (reverse (the list (list 1 "2"))))
+(mx (list string fixnum) (reverse (list 1 "2")))
+(mx (list fixnum string) (reverse (cons "1" (cons 2 '()))))
 
 (: f1 (forall (a) ((list-of a) -> a)))
 (define (f1 x) (car x))
@@ -301,3 +349,7 @@
 	    (fixnum 'not-ok)
 	    (else 'ok))))
 
+(assert ; clause order is respected
+ (compiler-typecase 1
+   (number #t)
+   (fixnum #f)))
